@@ -31,7 +31,9 @@ SEXP gcheck_vtrack(SEXP _vtrack, SEXP _envir)
 		parser.parse_exprs(exprs);
 	} catch (TGLException &e) {
 		rerror("%s", e.msg());
-	}
+    } catch (const bad_alloc &e) {
+        rerror("Out of memory");
+    }
 	return R_NilValue;
 }
 
