@@ -1,3 +1,5 @@
+#include <cstdint>
+#include <cmath>
 #include <unistd.h>
 
 #include "rdbutils.h"
@@ -13,12 +15,6 @@
 #include "GenomeTrackSparse.h"
 #include "TrackExpressionVars.h"
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
-#ifndef isnan
-#define isnan ::isnan
-#endif
-#endif
-
 const char *TrackExpressionVars::Track_var::FUNC_NAMES[TrackExpressionVars::Track_var::NUM_FUNCS] = {
 		"avg", "min", "max", "nearest", "stddev", "sum", "quantile", "global.percentile", "global.percentile.min", "global.percentile.max", "weighted.sum", "area"
 };
@@ -26,17 +22,22 @@ const char *TrackExpressionVars::Interv_var::FUNC_NAMES[TrackExpressionVars::Int
 
 using namespace rdb;
 
+
 TrackExpressionVars::TrackExpressionVars(rdb::IntervUtils &iu) :
 	m_iu(iu)
 {
-	m_imdfs1d.reserve(1000);
-	m_imdfs2d.reserve(1000);
-	m_track_n_imdfs.reserve(1000);
+	m_imdfs1d.reserve(10000);
+	m_imdfs2d.reserve(10000);
+	m_track_n_imdfs.reserve(10000);
 	m_groot = get_groot(m_iu.get_env());
 }
 
 TrackExpressionVars::~TrackExpressionVars()
 {
+	for (Track_vars::iterator ivar = m_track_vars.begin(); ivar != m_track_vars.end(); ivar++)
+		runprotect(ivar->rvar);
+	for (Interv_vars::iterator ivar = m_interv_vars.begin(); ivar != m_interv_vars.end(); ivar++)
+		runprotect(ivar->rvar);
 	for (Track_n_imdfs::iterator itrack_n_imdf = m_track_n_imdfs.begin(); itrack_n_imdf != m_track_n_imdfs.end(); ++itrack_n_imdf)
 		delete itrack_n_imdf->track;
 }
@@ -45,23 +46,25 @@ void TrackExpressionVars::parse_exprs(const vector<string> &track_exprs)
 {
 	enum { TRACK, VTRACK, NUM_VAR_TYPES };
 
-	SEXP rtracknames[NUM_VAR_TYPES];
+	SEXP rtracknames[NUM_VAR_TYPES] = { R_NilValue, R_NilValue };
 	SEXP gvtracks = R_NilValue;
 	SEXP vtracks = R_NilValue;
 
 	// retrieve track names
-	rprotect(rtracknames[TRACK] = findVar(install("GTRACKS"), m_iu.get_env()));
+	rprotect(rtracknames[TRACK] = findVar(install("GTRACKS"), findVar(install(".misha"), m_iu.get_env())));
+	SEXPCleaner rtracknames_track_cleaner(rtracknames[TRACK]);
 
 	// retrieve virtual track names (it's more complex since virtual track names are burried in a list of lists)
 	rtracknames[VTRACK] = R_NilValue;
-	rprotect(gvtracks = findVar(install("GVTRACKS"), m_iu.get_env()));
+	rprotect(gvtracks = findVar(install("GVTRACKS"), findVar(install(".misha"), m_iu.get_env())));
+	SEXPCleaner gvtracks_cleaner(gvtracks);
 
 	if (!isNull(gvtracks) && !isSymbol(gvtracks)) { 
 		SEXP gwds = getAttrib(gvtracks, R_NamesSymbol);
 
-		if (!isVector(gvtracks) || length(gvtracks) && !isString(gwds) || length(gwds) != length(gvtracks))
+		if (!isVector(gvtracks) || (length(gvtracks) && !isString(gwds)) || length(gwds) != length(gvtracks))
 			verror("Invalid format of GVTRACKS variable.\n"
-				   "To continue working with virtual tracks please remove this variable from the environment.");
+				"To continue working with virtual tracks please remove this variable from the environment.");
 
 		const char *gwd = get_gwd(m_iu.get_env());
 
@@ -70,9 +73,9 @@ void TrackExpressionVars::parse_exprs(const vector<string> &track_exprs)
 				vtracks = VECTOR_ELT(gvtracks, i);
 				SEXP vtracknames = getAttrib(vtracks, R_NamesSymbol);
 
-				if (!isVector(vtracks) || length(vtracks) && !isString(vtracknames) || length(vtracknames) != length(vtracks))
+				if (!isVector(vtracks) || (length(vtracks) && !isString(vtracknames)) || length(vtracknames) != length(vtracks))
 					verror("Invalid format of GVTRACKS variable.\n"
-				           "To continue working with virtual tracks please remove this variable from the environment.");
+						"To continue working with virtual tracks please remove this variable from the environment.");
 
 				rtracknames[VTRACK] = vtracknames;
 			}
@@ -86,7 +89,7 @@ void TrackExpressionVars::parse_exprs(const vector<string> &track_exprs)
 
 			for (int itrack = 0; itrack < length(rtracknames[var_type]); ++itrack) {
 				string track = CHAR(STRING_ELT(rtracknames[var_type], itrack));
-				size_t pos = 0;
+				uint64_t pos = 0;
 
 				while ((pos = iexpr->find(track, pos)) != string::npos) {
 					if (is_var(*iexpr, pos, pos + track.size())) {
@@ -275,7 +278,7 @@ void TrackExpressionVars::add_vtrack_var(const string &vtrack, SEXP rvtrack)
 	if (isString(rsrc) && length(rsrc) == 1) {
 		string track(CHAR(STRING_ELT(rsrc, 0)));
 
-		SEXP gtracks = findVar(install("GTRACKS"), m_iu.get_env());
+		SEXP gtracks = findVar(install("GTRACKS"), findVar(install(".misha"), m_iu.get_env()));
 		if (isString(gtracks)) {
 			for (int itrack = 0; itrack < length(gtracks); itrack++) {
 				if (!strcmp(CHAR(STRING_ELT(gtracks, itrack)), track.c_str())) {
@@ -284,7 +287,7 @@ void TrackExpressionVars::add_vtrack_var(const string &vtrack, SEXP rvtrack)
 				}
 			}
 		}
-	}
+        }
 
 	GIntervals intervs1d;
 	GIntervals2D intervs2d;
@@ -417,9 +420,9 @@ TrackExpressionVars::Track_var &TrackExpressionVars::add_vtrack_var_src_track(SE
 	int ifunc;
 	for (ifunc = 0; ifunc < Track_var::NUM_FUNCS; ++ifunc) {
 		if (!strcmp(func.c_str(), Track_var::FUNC_NAMES[ifunc])) {
-			if (GenomeTrack::is_1d(track_type) && (ifunc == Track_var::WEIGHTED_SUM || ifunc == Track_var::OCCUPIED_AREA) ||
-					GenomeTrack::is_2d(track_type) && (ifunc == Track_var::REG_NEAREST || ifunc == Track_var::STDDEV || ifunc == Track_var::SUM || ifunc == Track_var::QUANTILE) ||
-					track_type != GenomeTrack::FIXED_BIN && (ifunc == Track_var::PV || ifunc == Track_var::PV_MIN || ifunc == Track_var::PV_MAX))
+			if ((GenomeTrack::is_1d(track_type) && (ifunc == Track_var::WEIGHTED_SUM || ifunc == Track_var::OCCUPIED_AREA)) ||
+					(GenomeTrack::is_2d(track_type) && (ifunc == Track_var::REG_NEAREST || ifunc == Track_var::STDDEV || ifunc == Track_var::SUM || ifunc == Track_var::QUANTILE)) ||
+					(track_type != GenomeTrack::FIXED_BIN && (ifunc == Track_var::PV || ifunc == Track_var::PV_MIN || ifunc == Track_var::PV_MAX)))
 				verror("Virtual track %s: function %s is not supported by %s tracks", vtrack.c_str(), func.c_str(), GenomeTrack::TYPE_NAMES[track_type]);
 
 			if (ifunc == Track_var::QUANTILE) {
@@ -443,7 +446,7 @@ TrackExpressionVars::Track_var &TrackExpressionVars::add_vtrack_var_src_track(SE
 	}
 
 	if (ifunc >= Track_var::NUM_FUNCS)
-		verror("Virtual track %s: invalid function %s used for with a track", vtrack.c_str(), func.c_str());
+        verror("Virtual track %s: invalid function %s used for a virtual track", vtrack.c_str(), func.c_str());
 
 	return var;
 }
@@ -618,28 +621,30 @@ void TrackExpressionVars::init(const TrackExpressionIteratorBase &expr_itr)
 
 			if (access(pv_fname.c_str(), R_OK) < 0 && errno == ENOENT) {
 				char command[1000];
+				SEXP rretv;
 
-				Rprintf("Preparing track %s for percentiles queries\n", ivar->track_n_imdf->name.c_str());
-				sprintf(command,
+				REprintf("Preparing track %s for percentiles queries\n", ivar->track_n_imdf->name.c_str());
+				snprintf(command, sizeof(command),
 						"{ "
 						"	.ginteractive = getOption(\".ginteractive\")\n"
 						"	tryCatch({\n"
-						"			options(.ginteractive = F)\n"
-						"			.gtrack.prepare.pvals(\"%s\")\n"
+						"			options(.ginteractive = FALSE)\n"
+						"			misha:::.gtrack.prepare.pvals(\"%s\")\n"
 						"		},\n"
 						"		finally = { options(.ginteractive = .ginteractive) })"
 						" }",
 						ivar->track_n_imdf->name.c_str());
-				run_in_R(command, m_iu.get_env());
+				runprotect(rretv = run_in_R(command, m_iu.get_env()));
 			}
 
 			rprotect(val = RSaneUnserialize(pv_fname.c_str()));
+			SEXPCleaner val_cleaner(val);
 			SEXP breaks = getAttrib(val, install("breaks"));
 
 			if (breaks == R_NilValue || !isReal(breaks) || length(breaks) != length(val))
 				verror("File %s is in invalid format.", pv_fname.c_str());
 
-			pv.bins = REAL(val);
+			pv.bins.assign(REAL(val), REAL(val) + length(val));
 			pv.binfinder.init(REAL(breaks), length(breaks));
 		}
 	}
@@ -648,12 +653,12 @@ void TrackExpressionVars::init(const TrackExpressionIteratorBase &expr_itr)
 void TrackExpressionVars::define_r_vars(unsigned size)
 {
 	for (Track_vars::iterator ivar = m_track_vars.begin(); ivar != m_track_vars.end(); ivar++) {
-		rprotect(ivar->rvar = allocVector(REALSXP, size));
+		rprotect(ivar->rvar = RSaneAllocVector(REALSXP, size));
 		defineVar(install(ivar->var_name.c_str()), ivar->rvar, m_iu.get_env());
 		ivar->var = REAL(ivar->rvar);
 	}
 	for (Interv_vars::iterator ivar = m_interv_vars.begin(); ivar != m_interv_vars.end(); ivar++) {
-		rprotect(ivar->rvar = allocVector(REALSXP, size));
+		rprotect(ivar->rvar = RSaneAllocVector(REALSXP, size));
 		defineVar(install(ivar->var_name.c_str()), ivar->rvar, m_iu.get_env());
 		ivar->var = REAL(ivar->rvar);
 	}
@@ -665,15 +670,34 @@ void TrackExpressionVars::start_chrom(const GInterval &interval)
 		try {
 			string filename(track2path(m_iu.get_env(), itrack_n_imdf->name) + "/" + GenomeTrack::get_1d_filename(m_iu.get_chromkey(), interval.chromid));
 
-			delete itrack_n_imdf->track;
+            delete itrack_n_imdf->track;
 			if (itrack_n_imdf->type == GenomeTrack::FIXED_BIN) {
-				itrack_n_imdf->track = new GenomeTrackFixedBin;
-				((GenomeTrackFixedBin *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
+                itrack_n_imdf->track = new GenomeTrackFixedBin;
+                ((GenomeTrackFixedBin *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
 			} else if (itrack_n_imdf->type == GenomeTrack::SPARSE) {
 				itrack_n_imdf->track = new GenomeTrackSparse;
 				((GenomeTrackSparse *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
 			} else if (itrack_n_imdf->type == GenomeTrack::ARRAYS) {
 				itrack_n_imdf->track = new GenomeTrackArrays;
+
+				// Optimization step:
+				// Normally if two virtual tracks share the same source track, slice and iterator modifiers then only one real track
+				// (i.e. Track_n_imdfs) is created. Thus if two virtual tracks based on a common source array track differ only by slice
+				// still two Track_n_imdfs and subsequently two GenomeTrackArrays are created. These two GenomeTrackArrays objects will
+				// open exactly the same file, make exactly the same "seek" and read the same "file values" each time read_interval is called.
+				// Once "file values" are achieved each object will calculate its own "track value" based on its unique slice.
+				// We seek to prevent opening and reading the same file multiple times.
+				// set_master_obj() allows an object to use "file values" of another "master" object.
+				// Thus multiple reading of the same file is prevented, yet still the "track value" is calculated differently by each object.
+				for (Track_n_imdfs::iterator itrack_n_imdf2 = m_track_n_imdfs.begin(); itrack_n_imdf2 != itrack_n_imdf; ++itrack_n_imdf2) {
+					if (itrack_n_imdf->name == itrack_n_imdf2->name &&
+						itrack_n_imdf->imdf1d == itrack_n_imdf2->imdf1d && itrack_n_imdf->imdf2d == itrack_n_imdf2->imdf2d) 
+					{
+						((GenomeTrackArrays *)itrack_n_imdf->track)->set_master_obj((GenomeTrackArrays *)itrack_n_imdf2->track);
+						break;
+					}
+				}
+
 				((GenomeTrackArrays *)itrack_n_imdf->track)->init_read(filename.c_str(), interval.chromid);
 			} else
 				verror("Internal error: track %s of type %s is not supported by 1D iterators", itrack_n_imdf->name.c_str(), GenomeTrack::TYPE_NAMES[itrack_n_imdf->type]);
@@ -681,7 +705,7 @@ void TrackExpressionVars::start_chrom(const GInterval &interval)
 			verror("%s\n", e.msg());
 		}
 	}
-	register_track_functions();
+    register_track_functions();
 }
 
 void TrackExpressionVars::start_chrom(const GInterval2D &interval)
@@ -739,15 +763,16 @@ void TrackExpressionVars::start_chrom(const GInterval2D &interval)
 
 void TrackExpressionVars::set_vars(const GInterval &interval, unsigned idx)
 {
-	if (m_interval1d.chromid != interval.chromid)
-		start_chrom(interval);
+    if (m_interval1d.chromid != interval.chromid) {
+        start_chrom(interval);
+	}
 
 	m_interval1d = interval;
 
-	for (Iterator_modifiers1D::iterator iimdf = m_imdfs1d.begin(); iimdf != m_imdfs1d.end(); ++iimdf)
-		iimdf->transform(interval, m_iu.get_chromkey());
+    for (Iterator_modifiers1D::iterator iimdf = m_imdfs1d.begin(); iimdf != m_imdfs1d.end(); ++iimdf)
+        iimdf->transform(interval, m_iu.get_chromkey());
 
-	set_vars(idx);
+    set_vars(idx);
 }
 
 void TrackExpressionVars::set_vars(const GInterval2D &interval, const DiagonalBand &band, unsigned idx)
@@ -827,8 +852,7 @@ void TrackExpressionVars::set_vars(unsigned idx)
 
 				if (ivar->requires_pv) {
 					double val = ivar->var[idx];
-
-					if (!isnan(val)) {
+					if (!std::isnan(val)) {
 						int bin = ivar->pv_binned.binfinder.val2bin(val);
 						if (bin < 0) {
 							if (val <= ivar->pv_binned.binfinder.get_breaks().front())
@@ -981,3 +1005,4 @@ void TrackExpressionVars::set_vars(unsigned idx)
 		}
 	}
 }
+
